@@ -4,20 +4,12 @@
 
 #include "https_client.h"
 
-#include "mqtt_websockets/c-rbuf/include/ringbuffer.h"
-
 #include "aclk_util.h"
 
 #include "daemon/global_statistics.h"
 
 #define DEFAULT_CHUNKED_RESPONSE_BUFFER_SIZE (4096)
 
-enum http_parse_state {
-    HTTP_PARSE_INITIAL = 0,
-    HTTP_PARSE_HEADERS,
-    HTTP_PARSE_CONTENT
-};
-
 static const char *http_req_type_to_str(http_req_type_t req) {
     switch (req) {
         case HTTP_REQ_GET:
@@ -33,39 +25,33 @@ static const char *http_req_type_to_str(http_req_type_t req) {
 
 #define TRANSFER_ENCODING_CHUNKED (-2)
 
-typedef struct {
-    enum http_parse_state state;
-    int content_length;
-    int http_code;
-
-    // for chunked data only
-    char *chunked_response;
-    size_t chunked_response_size;
-    size_t chunked_response_written;
-
-    enum chunked_content_state {
-        CHUNKED_CONTENT_CHUNK_SIZE = 0,
-        CHUNKED_CONTENT_CHUNK_DATA,
-        CHUNKED_CONTENT_CHUNK_END_CRLF,
-        CHUNKED_CONTENT_FINAL_CRLF
-    } chunked_content_state;
-
-    size_t chunk_size;
-    size_t chunk_got;
-} http_parse_ctx;
-
 #define HTTP_PARSE_CTX_INITIALIZER { .state = HTTP_PARSE_INITIAL, .content_length = -1, .http_code = 0 }
-static inline void http_parse_ctx_clear(http_parse_ctx *ctx) {
+void http_parse_ctx_create(http_parse_ctx *ctx)
+{
     ctx->state = HTTP_PARSE_INITIAL;
     ctx->content_length = -1;
     ctx->http_code = 0;
+    ctx->headers = c_rhash_new(0);
+    ctx->flags = HTTP_PARSE_FLAGS_DEFAULT;
+}
+
+void http_parse_ctx_destroy(http_parse_ctx *ctx)
+{
+    c_rhash_iter_t iter;
+    const char *key;
+
+    c_rhash_iter_t_initialize(&iter);
+    while ( !c_rhash_iter_str_keys(ctx->headers, &iter, &key) ) {
+        void *val;
+        c_rhash_get_ptr_by_str(ctx->headers, key, &val);
+        freez(val);
+    }
+
+    c_rhash_destroy(ctx->headers);
 }
 
 #define POLL_TO_MS 100
 
-#define NEED_MORE_DATA  0
-#define PARSE_SUCCESS   1
-#define PARSE_ERROR    -1
 #define HTTP_LINE_TERM "\x0D\x0A"
 #define RESP_PROTO "HTTP/1.1 "
 #define HTTP_KEYVAL_SEPARATOR ": "
@@ -76,7 +62,7 @@ static int process_http_hdr(http_parse_ctx *parse_ctx, const char *key, const ch
 {
     // currently we care only about specific headers
     // we can skip the rest
-    if (!strcmp("content-length", key)) {
+    if (parse_ctx->content_length < 0 && !strcmp("content-length", key)) {
         if (parse_ctx->content_length == TRANSFER_ENCODING_CHUNKED) {
             netdata_log_error("Content-length and transfer-encoding: chunked headers are mutually exclusive");
             return 1;
@@ -85,7 +71,7 @@ static int process_http_hdr(http_parse_ctx *parse_ctx, const char *key, const ch
             netdata_log_error("Duplicate content-length header");
             return 1;
         }
-        parse_ctx->content_length = atoi(val);
+        parse_ctx->content_length = str2u(val);
         if (parse_ctx->content_length < 0) {
             netdata_log_error("Invalid content-length %d", parse_ctx->content_length);
             return 1;
@@ -102,9 +88,20 @@ static int process_http_hdr(http_parse_ctx *parse_ctx, const char *key, const ch
         }
         return 0;
     }
+    char *val_cpy = strdupz(val);
+    c_rhash_insert_str_ptr(parse_ctx->headers, key, val_cpy);
     return 0;
 }
 
+const char *get_http_header_by_name(http_parse_ctx *ctx, const char *name)
+{
+    const char *ret;
+    if (c_rhash_get_ptr_by_str(ctx->headers, name, (void**)&ret))
+        return NULL;
+
+    return ret;
+}
+
 static int parse_http_hdr(rbuf_t buf, http_parse_ctx *parse_ctx)
 {
     int idx, idx_end;
@@ -169,8 +166,8 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
             case CHUNKED_CONTENT_CHUNK_SIZE:
                 if (!rbuf_find_bytes(buf, HTTP_LINE_TERM, strlen(HTTP_LINE_TERM), &idx)) {
                     if (rbuf_bytes_available(buf) >= rbuf_get_capacity(buf))
-                        return PARSE_ERROR;
-                    return NEED_MORE_DATA;
+                        return HTTP_PARSE_ERROR;
+                    return HTTP_PARSE_NEED_MORE_DATA;
                 }
                 if (idx == 0) {
                     parse_ctx->chunked_content_state = CHUNKED_CONTENT_FINAL_CRLF;
@@ -178,7 +175,7 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
                 }
                 if (idx >= HTTP_HDR_BUFFER_SIZE) {
                     netdata_log_error("Chunk size is too long");
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 }
                 char buf_size[HTTP_HDR_BUFFER_SIZE];
                 rbuf_pop(buf, buf_size, idx);
@@ -186,13 +183,13 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
                 long chunk_size = strtol(buf_size, NULL, 16);
                 if (chunk_size < 0 || chunk_size == LONG_MAX) {
                     netdata_log_error("Chunk size out of range");
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 }
                 parse_ctx->chunk_size = chunk_size;
                 if (parse_ctx->chunk_size == 0) {
                     if (errno == EINVAL) {
                         netdata_log_error("Invalid chunk size");
-                        return PARSE_ERROR;
+                        return HTTP_PARSE_ERROR;
                     }
                     parse_ctx->chunked_content_state = CHUNKED_CONTENT_CHUNK_END_CRLF;
                     continue;
@@ -204,7 +201,7 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
                 // fallthrough
             case CHUNKED_CONTENT_CHUNK_DATA:
                 if (!(bytes_to_copy = rbuf_bytes_available(buf)))
-                    return NEED_MORE_DATA;
+                    return HTTP_PARSE_NEED_MORE_DATA;
                 if (bytes_to_copy > parse_ctx->chunk_size - parse_ctx->chunk_got)
                     bytes_to_copy = parse_ctx->chunk_size - parse_ctx->chunk_got;
                 rbuf_pop(buf, parse_ctx->chunked_response + parse_ctx->chunked_response_written, bytes_to_copy);
@@ -217,19 +214,19 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
             case CHUNKED_CONTENT_FINAL_CRLF:
             case CHUNKED_CONTENT_CHUNK_END_CRLF:
                 if (rbuf_bytes_available(buf) < strlen(HTTP_LINE_TERM))
-                    return NEED_MORE_DATA;
+                    return HTTP_PARSE_NEED_MORE_DATA;
                 char buf_crlf[strlen(HTTP_LINE_TERM)];
                 rbuf_pop(buf, buf_crlf, strlen(HTTP_LINE_TERM));
                 if (memcmp(buf_crlf, HTTP_LINE_TERM, strlen(HTTP_LINE_TERM))) {
                     netdata_log_error("CRLF expected");
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 }
                 if (parse_ctx->chunked_content_state == CHUNKED_CONTENT_FINAL_CRLF) {
                     if (parse_ctx->chunked_response_size != parse_ctx->chunked_response_written)
                         netdata_log_error("Chunked response size mismatch");
                     chunked_response_buffer_grow_by(parse_ctx, 1);
                     parse_ctx->chunked_response[parse_ctx->chunked_response_written] = 0;
-                    return PARSE_SUCCESS;
+                    return HTTP_PARSE_SUCCESS;
                 }
                 if (parse_ctx->chunk_size == 0) {
                     parse_ctx->chunked_content_state = CHUNKED_CONTENT_FINAL_CRLF;
@@ -241,34 +238,34 @@ static int process_chunked_content(rbuf_t buf, http_parse_ctx *parse_ctx)
     } while(1);
 }
 
-static int parse_http_response(rbuf_t buf, http_parse_ctx *parse_ctx)
+http_parse_rc parse_http_response(rbuf_t buf, http_parse_ctx *parse_ctx)
 {
     int idx;
     char rc[4];
 
     do {
         if (parse_ctx->state != HTTP_PARSE_CONTENT && !rbuf_find_bytes(buf, HTTP_LINE_TERM, strlen(HTTP_LINE_TERM), &idx))
-            return NEED_MORE_DATA;
+            return HTTP_PARSE_NEED_MORE_DATA;
         switch (parse_ctx->state) {
             case HTTP_PARSE_INITIAL:
                 if (rbuf_memcmp_n(buf, RESP_PROTO, strlen(RESP_PROTO))) {
                     netdata_log_error("Expected response to start with \"%s\"", RESP_PROTO);
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 }
                 rbuf_bump_tail(buf, strlen(RESP_PROTO));
                 if (rbuf_pop(buf, rc, 4) != 4) {
                     netdata_log_error("Expected HTTP status code");
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 }
                 if (rc[3] != ' ') {
                     netdata_log_error("Expected space after HTTP return code");
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 }
                 rc[3] = 0;
                 parse_ctx->http_code = atoi(rc);
                 if (parse_ctx->http_code < 100 || parse_ctx->http_code >= 600) {
                     netdata_log_error("HTTP code not in range 100 to 599");
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 }
 
                 rbuf_find_bytes(buf, HTTP_LINE_TERM, strlen(HTTP_LINE_TERM), &idx);
@@ -284,7 +281,7 @@ static int parse_http_response(rbuf_t buf, http_parse_ctx *parse_ctx)
                     break;
                 }
                 if (parse_http_hdr(buf, parse_ctx))
-                    return PARSE_ERROR;
+                    return HTTP_PARSE_ERROR;
                 rbuf_find_bytes(buf, HTTP_LINE_TERM, strlen(HTTP_LINE_TERM), &idx);
                 rbuf_bump_tail(buf, idx + strlen(HTTP_LINE_TERM));
                 break;
@@ -294,11 +291,14 @@ static int parse_http_response(rbuf_t buf, http_parse_ctx *parse_ctx)
                     return process_chunked_content(buf, parse_ctx);
 
                 if (parse_ctx->content_length < 0)
-                    return PARSE_SUCCESS;
+                    return HTTP_PARSE_SUCCESS;
+
+                if (parse_ctx->flags & HTTP_PARSE_FLAG_DONT_WAIT_FOR_CONTENT)
+                    return HTTP_PARSE_SUCCESS;
 
                 if (rbuf_bytes_available(buf) >= (size_t)parse_ctx->content_length)
-                    return PARSE_SUCCESS;
-                return NEED_MORE_DATA;
+                    return HTTP_PARSE_SUCCESS;
+                return HTTP_PARSE_NEED_MORE_DATA;
         }
     } while(1);
 }
@@ -486,7 +486,7 @@ static int read_parse_response(https_req_ctx_t *ctx) {
         } while (ctx->poll_fd.events == 0 && rbuf_bytes_free(ctx->buf_rx) > 0);
     } while (!(ret = parse_http_response(ctx->buf_rx, &ctx->parse_ctx)));
 
-    if (ret != PARSE_SUCCESS) {
+    if (ret != HTTP_PARSE_SUCCESS) {
         netdata_log_error("Error parsing HTTP response");
         return 1;
     }
@@ -500,7 +500,7 @@ static int handle_http_request(https_req_ctx_t *ctx) {
     BUFFER *hdr = buffer_create(TX_BUFFER_SIZE, &netdata_buffers_statistics.buffers_aclk);
     int rc = 0;
 
-    http_parse_ctx_clear(&ctx->parse_ctx);
+    http_parse_ctx_create(&ctx->parse_ctx);
 
     // Prepare data to send
     switch (ctx->request->request_type) {
@@ -526,7 +526,7 @@ static int handle_http_request(https_req_ctx_t *ctx) {
         buffer_strcat(hdr, ctx->request->url);
     }
 
-    buffer_strcat(hdr, " HTTP/1.1\x0D\x0A");
+    buffer_strcat(hdr, HTTP_1_1 HTTP_ENDL);
 
     //TODO Headers!
     if (ctx->request->request_type != HTTP_REQ_CONNECT) {
@@ -661,12 +661,15 @@ int https_request(https_req_t *request, https_req_response_t *response) {
         ctx->request = &req;
         if (handle_http_request(ctx)) {
             netdata_log_error("Failed to CONNECT with proxy");
+            http_parse_ctx_destroy(&ctx->parse_ctx);
             goto exit_sock;
         }
         if (ctx->parse_ctx.http_code != 200) {
             netdata_log_error("Proxy didn't return 200 OK (got %d)", ctx->parse_ctx.http_code);
+            http_parse_ctx_destroy(&ctx->parse_ctx);
             goto exit_sock;
         }
+        http_parse_ctx_destroy(&ctx->parse_ctx);
         netdata_log_info("Proxy accepted CONNECT upgrade");
     }
     ctx->request = request;
@@ -713,8 +716,10 @@ int https_request(https_req_t *request, https_req_response_t *response) {
     // The actual request here
     if (handle_http_request(ctx)) {
         netdata_log_error("Couldn't process request");
+        http_parse_ctx_destroy(&ctx->parse_ctx);
         goto exit_SSL;
     }
+    http_parse_ctx_destroy(&ctx->parse_ctx);
     response->http_code = ctx->parse_ctx.http_code;
     if (ctx->parse_ctx.content_length == TRANSFER_ENCODING_CHUNKED) {
         response->payload_size = ctx->parse_ctx.chunked_response_size;
