@@ -401,6 +401,11 @@
 
 #define NETDATA_CHART_PRIO_STATSD_PRIVATE            90000 // many charts
 
+// Logs Management
+
+#define NETDATA_CHART_PRIO_LOGS_BASE                 95000 // many charts
+#define NETDATA_CHART_PRIO_LOGS_STATS_BASE           160000 // logsmanagement stats in "Netdata Monitoring"
+
 // PCI
 
 #define NETDATA_CHART_PRIO_PCI_AER                  100000
