@@ -13,7 +13,7 @@
 #define NETDATA_DC_REQUEST_NOT_CACHE_CHART "dc_not_cache"
 #define NETDATA_DC_REQUEST_NOT_FOUND_CHART "dc_not_found"
 
-#define NETDATA_DIRECTORY_CACHE_SUBMENU "directory cache (eBPF)"
+#define NETDATA_DIRECTORY_CACHE_SUBMENU "directory cache"
 
 // configuration file
 #define NETDATA_DIRECTORY_DCSTAT_CONFIG_FILE "dcstat.conf"
