@@ -89,21 +89,21 @@ static size_t disks_added = 0, disks_found = 0;
 
 static void disk_free(struct disk *dm) {
     if (likely(dm->st_io))
-        rrdset_is_obsolete(dm->st_io);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_io);
     if (likely(dm->st_ops))
-        rrdset_is_obsolete(dm->st_ops);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_ops);
     if (likely(dm->st_qops))
-        rrdset_is_obsolete(dm->st_qops);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_qops);
     if (likely(dm->st_util))
-        rrdset_is_obsolete(dm->st_util);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_util);
     if (likely(dm->st_iotime))
-        rrdset_is_obsolete(dm->st_iotime);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_iotime);
     if (likely(dm->st_await))
-        rrdset_is_obsolete(dm->st_await);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_await);
     if (likely(dm->st_avagsz))
-        rrdset_is_obsolete(dm->st_avagsz);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_avagsz);
     if (likely(dm->st_svctm))
-        rrdset_is_obsolete(dm->st_svctm);
+        rrdset_is_obsolete___safe_from_collector_thread(dm->st_svctm);
 
     disks_added--;
     freez(dm->name);
