@@ -16,14 +16,11 @@
 #define PLUGINSD_KEYWORD_DYNCFG_ENABLE          "DYNCFG_ENABLE"
 #define PLUGINSD_KEYWORD_DYNCFG_REGISTER_MODULE "DYNCFG_REGISTER_MODULE"
 #define PLUGINSD_KEYWORD_DYNCFG_REGISTER_JOB    "DYNCFG_REGISTER_JOB"
+#define PLUGINSD_KEYWORD_DYNCFG_RESET           "DYNCFG_RESET"
 
 #define PLUGINSD_KEYWORD_REPORT_JOB_STATUS      "REPORT_JOB_STATUS"
 #define PLUGINSD_KEYWORD_DELETE_JOB             "DELETE_JOB"
 
-#define PLUGINSD_LINE_MAX_SSL_READ 512
-
-#define PLUGINSD_MAX_WORDS 20
-
 #define PLUGINSD_MAX_DIRECTORIES 20
 extern char *plugin_directories[PLUGINSD_MAX_DIRECTORIES];
 
