@@ -59,7 +59,8 @@ void netdev_rename_device_add(
     const char *container_device,
     const char *container_name,
     RRDLABELS *labels,
-    const char *ctx_prefix);
+    const char *ctx_prefix,
+    const DICTIONARY_ITEM *cgroup_netdev_link);
 
 void netdev_rename_device_del(const char *host_device);
 
