@@ -920,22 +920,14 @@ static void xenstat_send_domain_metrics() {
 }
 
 int main(int argc, char **argv) {
-    stderror = stderr;
     clocks_init();
 
     // ------------------------------------------------------------------------
     // initialization of netdata plugin
 
-    program_name = "xenstat.plugin";
+    program_name = PLUGIN_XENSTAT_NAME;
 
-    // disable syslog
-    error_log_syslog = 0;
-
-    // set errors flood protection to 100 logs per hour
-    error_log_errors_per_period = 100;
-    error_log_throttle_period = 3600;
-
-    log_set_global_severity_for_external_plugins();
+    nd_log_initialize_for_external_plugins(PLUGIN_XENSTAT_NAME);
 
     // ------------------------------------------------------------------------
     // parse command line parameters
