@@ -37,6 +37,7 @@ int getsysctl(const char *name, int *mib, size_t miblen, void *ptr, size_t *len)
 #if __APPLE__
 
 #include <sys/sysctl.h>
+#include "endian.h"
 
 #define GETSYSCTL_BY_NAME(name, var) getsysctl_by_name(name, &(var), sizeof(var))
 int getsysctl_by_name(const char *name, void *ptr, size_t len);
@@ -48,8 +49,10 @@ int getsysctl_by_name(const char *name, void *ptr, size_t len);
 
 extern const char *os_type;
 
-extern int processors;
-long get_system_cpus(void);
+#define get_system_cpus() get_system_cpus_with_cache(true, false)
+#define get_system_cpus_uncached() get_system_cpus_with_cache(false, false)
+long get_system_cpus_with_cache(bool cache, bool for_netdata);
+unsigned long read_cpuset_cpus(const char *filename, long system_cpus);
 
 extern pid_t pid_max;
 pid_t get_system_pid_max(void);
