@@ -3,14 +3,14 @@
 
 # shellcheck source=packaging/makeself/functions.sh
 . "$(dirname "${0}")/../functions.sh" "${@}" || exit 1
-
-version='1.3'
+# Source of truth for all the packages we bundle in static builds
+. "$(dirname "${0}")/../bundled-packages" || exit 1
 
 # shellcheck disable=SC2015
 [ "${GITHUB_ACTIONS}" = "true" ] && echo "::group::Building ioping" || true
 
-fetch "ioping-${version}" "https://github.com/koct9i/ioping/archive/v${version}.tar.gz" \
-    7aa48e70aaa766bc112dea57ebbe56700626871052380709df3a26f46766e8c8 ioping
+fetch "ioping-${IOPING_VERSION}" "${IOPING_SOURCE}/archive/refs/tags/v${IOPING_VERSION}.tar.gz" \
+    "${IOPING_ARTIFACT_SHA256}" ioping
 
 export CFLAGS="-static -pipe"
 
@@ -22,7 +22,7 @@ fi
 run mkdir -p "${NETDATA_INSTALL_PATH}"/usr/libexec/netdata/plugins.d/
 run install -o root -g root -m 4750 ioping "${NETDATA_INSTALL_PATH}"/usr/libexec/netdata/plugins.d/
 
-store_cache ioping "${NETDATA_MAKESELF_PATH}/tmp/ioping-${version}"
+store_cache ioping "${NETDATA_MAKESELF_PATH}/tmp/ioping-${IOPING_VERSION}"
 
 if [ "${NETDATA_BUILD_WITH_DEBUG}" -eq 0 ]; then
   run strip "${NETDATA_INSTALL_PATH}"/usr/libexec/netdata/plugins.d/ioping
